@@ -2,6 +2,7 @@ from collections import defaultdict
 import boto3
 import pendulum
 import pandas as pd
+import awswrangler as wr
 from logging import getLogger
 from botocore.exceptions import ProfileNotFound, ClientError
 from sumatra.config import CONFIG
@@ -31,6 +32,11 @@ class AdminClient:
                 f"Error connecting with '{aws_profile}' as the AWS profile for {CONFIG.instance}: {e}"
             )
 
+    def query_athena(self, tenant, sql):
+        return wr.athena.read_sql_query(
+            workgroup=TENANT_PREFIX + tenant, database=TENANT_PREFIX + tenant, sql=sql
+        )
+
     def _tenants_from_groups(self, resp):
         tenants = []
         for group in resp["Groups"]:
