@@ -41,6 +41,19 @@ def _print_diff(diff, no_color=False) -> None:
         modified = [(f, "~") for f in tdiff["featuresRedefined"]]
         total_modified += len(modified)
         tdiffs[event_type] = added + deleted + modified
+    tables_added, tables_removed, tables_updated = [], [], []
+    for tblDiff in diff["tableDiffs"]:
+        name = tblDiff["id"]
+        if tblDiff["oldVersion"] == "" and tblDiff["newVersion"] == "":
+            raise RuntimeError(f"found empty old and new versions for table: '{name}'")
+        if tblDiff["oldVersion"] == tblDiff["newVersion"]:
+            raise RuntimeError(f"found same old and new versions for table: '{name}'")
+        if tblDiff["oldVersion"] == "":
+            tables_added.append(name)
+        elif tblDiff["newVersion"] == "":
+            tables_removed.append(name)
+        else:
+            tables_updated.append(name)
 
     color = {"+": Fore.GREEN, "-": Fore.RED, "~": Fore.YELLOW}
     for event in sorted(events_added + events_deleted + events_modified):
@@ -58,6 +71,17 @@ def _print_diff(diff, no_color=False) -> None:
             else:
                 print(f"  {color[op] + op + ' ' + f + Fore.RESET}")
         print()
+    for tbl in tables_added + tables_removed + tables_updated:
+        op = "+" if tbl in tables_added else "-" if tbl in tables_removed else "~"
+        if no_color:
+            print(f"{op} table {tbl}")
+        else:
+            print(
+                f"{Style.BRIGHT + color[op] + op} table {tbl + Fore.RESET + Style.NORMAL}"
+            )
+    if tables_added + tables_removed + tables_updated:
+        print()
+
     plan = ", ".join(
         s
         for s in [
@@ -66,6 +90,9 @@ def _print_diff(diff, no_color=False) -> None:
             _quantify("add {n} feature{plural}", total_added),
             _quantify("delete {n} feature{plural}", total_deleted),
             _quantify("modify {n} feature{plural}", total_modified),
+            _quantify("add {n} table{plural}", len(tables_added)),
+            _quantify("remove {n} table{plural}", len(tables_removed)),
+            _quantify("update {n} table{plural}", len(tables_updated)),
         ]
         if s
     )
@@ -93,6 +120,7 @@ def login(args) -> None:
         if not instance:
             instance = curr_instance
     CONFIG.instance = instance
+    CONFIG.update_from_stack()
     CONFIG.save(update_default_instance=True)
 
     auth = CognitoAuth()
@@ -106,13 +134,16 @@ def plan(args) -> None:
     print(f"Comparing '{scowl_dir}' to LIVE on {CONFIG.instance} ({client.tenant()})\n")
     branch = _tmp_branch_name()
     try:
-        client.create_branch_from_dir(scowl_dir, branch)
+        client.create_branch_from_dir(scowl_dir, branch, args.deps_file)
         diff = client.diff_branch_with_live(branch)
         _print_diff(diff, args.no_color)
         if not any(diff.values()):
             sys.exit(0)
     finally:
-        client.delete_branch(branch)
+        try:
+            client.delete_branch(branch)
+        except:
+            pass
 
 
 def apply(args):
@@ -138,12 +169,19 @@ def apply(args):
             print("Aborting.")
             sys.exit(1)
 
-    Client().publish_dir(args.scowl_dir)
+    Client().publish_dir(args.scowl_dir, args.deps_file)
     print("Successfully published to LIVE.")
 
 
+def pull(args):
+    branch = Client().save_branch_to_dir(args.scowl_dir, args.branch, args.deps_file)
+    print(f"Successfully pulled branch '{branch}'.")
+
+
 def push(args):
-    branch = Client().create_branch_from_dir(args.scowl_dir, args.branch)
+    branch = Client().create_branch_from_dir(
+        args.scowl_dir, args.branch, args.deps_file
+    )
     print(f"Successfully pushed to branch '{branch}'.")
 
 
@@ -170,7 +208,7 @@ def timeline_show(args):
 
 
 def timeline_upload(args):
-    Client().create_timeline_from_file(args.file, args.timeline)
+    Client().create_timeline_from_file(args.timeline, args.file)
     print(f"Successfully uploaded to timeline '{args.timeline}'.")
 
 
@@ -183,8 +221,8 @@ def branch_list(args):
 
 
 def branch_delete(args):
-    deleted = Client().delete_branch(args.branch)
-    print(f"Successfully deleted branch '{deleted}'.")
+    Client().delete_branch(args.branch)
+    print(f"Successfully deleted branch '{args.branch}'.")
 
 
 def branch_show(args):
@@ -196,6 +234,44 @@ def branch_select(args):
     print(f"Default branch config updated to '{args.branch}' for {CONFIG.instance}\n")
 
 
+def deps_update(args):
+    deps_file = Client().save_deps(args.live, args.deps_file)
+    print(f"Saved updated dependencies to '{deps_file}'")
+
+
+def deps_list(args):
+    print(Client().get_deps(args.live))
+
+
+def deps_resolve(args):
+    print(Client().resolve_deps_from_file(args.deps_file))
+
+
+def table_list(args):
+    tables = Client().get_tables().sort_index()
+    if args.show:
+        print(tables)
+    else:
+        print("\n".join(tables.index.to_list()))
+
+
+def table_delete(args):
+    Client().delete_table(args.table)
+    print(f"Successfully deleted table '{args.table}'")
+
+
+def table_show(args):
+    print(Client().get_tables().loc[args.table])
+
+
+def table_schema(args):
+    print(Client().get_table_schema(args.table, args.version))
+
+
+def table_history(args):
+    print(Client().get_table_history(args.table))
+
+
 def version(args):
     print(Client().version())
 
@@ -315,6 +391,52 @@ BRANCH_COMMANDS = [
     },
 ]
 
+DEPS_COMMANDS = [
+    {
+        "name": "update",
+        "help": "fetch and save deps to file",
+        "handler": deps_update,
+    },
+    {
+        "name": "list",
+        "help": "fetch and print deps to stdout",
+        "handler": deps_list,
+    },
+    {
+        "name": "resolve",
+        "help": "fetch metadata for local deps and print to stdout",
+        "handler": deps_resolve,
+    },
+]
+
+TABLE_COMMANDS = [
+    {
+        "name": "list",
+        "help": "list all tables",
+        "handler": table_list,
+    },
+    {
+        "name": "delete",
+        "help": "delete table",
+        "handler": table_delete,
+    },
+    {
+        "name": "show",
+        "help": "display table metadata",
+        "handler": table_show,
+    },
+    {
+        "name": "schema",
+        "help": "display table schema",
+        "handler": table_schema,
+    },
+    {
+        "name": "history",
+        "help": "display table history",
+        "handler": table_history,
+    },
+]
+
 COMMANDS = [
     {
         "name": "login",
@@ -341,6 +463,21 @@ COMMANDS = [
         "help": "run `sumatra branch -h` for subcommands",
         "subcommands": BRANCH_COMMANDS,
     },
+    {
+        "name": "pull",
+        "help": "save remote branch scowl to local dir",
+        "handler": pull,
+    },
+    {
+        "name": "deps",
+        "help": "run `sumatra deps -h` for subcommands",
+        "subcommands": DEPS_COMMANDS,
+    },
+    {
+        "name": "table",
+        "help": "run `sumatra table -h` for subcommands",
+        "subcommands": TABLE_COMMANDS,
+    },
     {
         "name": "timeline",
         "help": "run `sumatra timeline -h` for subcommands",
@@ -381,8 +518,9 @@ def main():
         if "handler" in cmd:
             p.set_defaults(handler=cmd["handler"])
 
-        if cmd["name"] in ["plan", "apply", "push"]:
+        if cmd["name"] in ["plan", "apply", "push", "pull"]:
             p.add_argument("--scowl-dir", metavar="DIR", help="path to scowl files")
+            p.add_argument("--deps-file", metavar="FILE", help="path to deps file")
 
         if cmd["name"] in ["plan", "apply"]:
             p.add_argument(
@@ -396,7 +534,7 @@ def main():
                 help="automatically agree to all prompts",
             )
 
-        if cmd["name"] == "push":
+        if cmd["name"] in ["push", "pull"]:
             p.add_argument("--branch", metavar="NAME", help="remote branch name")
 
         if cmd["name"] == "admin":
@@ -425,6 +563,28 @@ def main():
                         p.add_argument(
                             "--show", action="store_true", help="display metadata"
                         )
+                if cmd["name"] == "deps":
+                    if subcmd["name"] in ["update", "list"]:
+                        p.add_argument(
+                            "--live",
+                            action="store_true",
+                            help="fetch live versions instead of latest",
+                        )
+                    if subcmd["name"] in ["update", "resolve"]:
+                        p.add_argument(
+                            "--deps-file", metavar="FILE", help="path to deps file"
+                        )
+                if cmd["name"] == "table":
+                    if subcmd["name"] in ["delete", "show", "schema", "history"]:
+                        p.add_argument("table", help="table name")
+                    if subcmd["name"] == "list":
+                        p.add_argument(
+                            "--show", action="store_true", help="display metadata"
+                        )
+                    if subcmd["name"] == "schema":
+                        p.add_argument(
+                            "--version", metavar="ID", help="specify table version"
+                        )
                 if cmd["name"] == "timeline":
                     if subcmd["name"] in ["delete", "show", "upload", "schema"]:
                         p.add_argument("timeline", help="remote timeline name")
