@@ -317,7 +317,8 @@ class Client:
             pass
 
         scowls = _load_scowl_files(scowl_dir)
-        scowls[DEPS_FILE] = open(deps_file).read()
+        if os.path.exists(deps_file):
+            scowls[DEPS_FILE] = open(deps_file).read()
         if not scowls:
             raise RuntimeError(
                 f"Unable to push local dir. '{scowl_dir}' has no .scowl files."
@@ -1872,7 +1873,10 @@ class Client:
         """
         query = """
             mutation DeleteTableVersion($name: String!, $version: String!) {
-                deleteTableVersion(name: $name, version: $version)
+                deleteTableVersion(name: $name, version: $version) {
+                    name
+                    version
+                }
             }
         """
         ret = self._gql_client.execute(
@@ -1896,7 +1900,9 @@ class Client:
         """
         query = """
             mutation DeleteTable($name: String!) {
-                deleteTable(name: $name)
+                deleteTable(name: $name) {
+                    name
+                }
             }
         """
         ret = self._gql_client.execute(
